# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
class Constants:
    # well-known environment variables
    AZURE_AI_PROJECT_ENDPOINT = "AZURE_AI_PROJECT_ENDPOINT"
    AGENT_ID = "AGENT_ID"
    AGENT_NAME = "AGENT_NAME"
    AGENT_PROJECT_RESOURCE_ID = "AGENT_PROJECT_NAME"
    OTEL_EXPORTER_ENDPOINT = "OTEL_EXPORTER_ENDPOINT"
    OTEL_EXPORTER_OTLP_PROTOCOL = "OTEL_EXPORTER_OTLP_PROTOCOL"
    AGENT_LOG_LEVEL = "AGENT_LOG_LEVEL"
    AGENT_DEBUG_ERRORS = "AGENT_DEBUG_ERRORS"
    ENABLE_APPLICATION_INSIGHTS_LOGGER = "AGENT_APP_INSIGHTS_ENABLED"
    AZURE_AI_WORKSPACE_ENDPOINT = "AZURE_AI_WORKSPACE_ENDPOINT"
    AZURE_AI_TOOLS_ENDPOINT = "AZURE_AI_TOOLS_ENDPOINT"
